

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <cstdlib>
#include <cstring>
#include <vector>
#include <string>
#include <fstream>
#include <sstream>

#include "psoptimizer.h"
#include "rpsoptimizer.h"

using namespace std;

enum
{
  NO_TO_ALL=-2,
  NO,
  ASK_USER,
  YES,
  YES_TO_ALL
};

vector<string> &Parse( const string &str, vector<string> &params );
FILE *open_file( string path, int *mode );

int main(int argc, char *argv[])
{
//   char *polje[]={"ime","runfile","pso","res/asdf"};
//   argc=4;
//   argv=polje;
  if( argc<4 || (strcmp( "pso", argv[2] ) && strcmp( "rpso", argv[2] )) )
  {
    cout<<"Način korištenja:"<<endl;
    cout<<argv[0]<<" <ulazna_datoteka> <pso|rpso> <izlazna_datoteka>"<<endl;
    return 1;
  }
  
  ifstream file;
  file.open( argv[1], ifstream::in );
  
  if( !file.is_open() )
  {
    cout<<"Ne mogu otvoriti datoteku \""<<argv[1]<<"\""<<endl;
    cout<<"Provjerite postoji li datoteka i imate li prava pristupa"<<endl;
    return 2;
  }
  
  FILE *fp;
  int mode=ASK_USER;
  int counterSF6=0;
  int counterDJS=0;
  int counterGW=0;
  int counterRAS=0;
  int counterRB=0;
  
  /*
  * SF6 - Schaffer F6
  * DJS - De Jongs' Sphere
  * GW  - Griewangk
  * RAS - Rastrigin
  * RB  - Rosenbrock
  */

  
  
  ParticleSwarmOptimizer *optimizer=NULL;
  string line;
  vector<string> params;
  
  // program flow
  while( file.good() )
  {
    // housework
    if( !strcmp( "pso", argv[2] ) ) optimizer=new PSOptimizer;
    else optimizer=new RPSOptimizer;
    if( !optimizer )
    {
      cout<<"Greška kod alokacije memorije! Žao mi je..."<<endl;
      return -1;
    }
    
    
    getline( file, line );
    
    Parse( line, params );
    
    // run optimization based on parsed input
    // housework
    if( params.empty() || params.at(0).empty() || params.at(0).at(0)=='#' ) continue;
    
    
    // check for supported codenames
    if( params.at(0).compare("SF6") &&
        params.at(0).compare("DJS") &&
        params.at(0).compare("GW") &&
        params.at(0).compare("RAS") &&
        params.at(0).compare("RB") )
    {
      //cout<<"Problem s kodnom oznakom \""<<params.at(0)<<"\" nije podržan"<<endl;
      continue;
    }
    
    string path;
    // setting parameters
    try
    {
        optimizer->SetDimension( atoi( params.at(1).c_str() ) );
        optimizer->SetSwarmSize( atoi( params.at(2).c_str() ) );
        optimizer->SetMaxX( atof( params.at(3).c_str() ) );
        optimizer->SetMaxV( atof( params.at(4).c_str() ) );
        int tmp=atoi( params.at(5).c_str() );
        optimizer->SetMaxIterations( tmp );
        optimizer->SetCutoff( atof( params.at(6).c_str() ) );
        optimizer->SetInitRang( atof( params.at(7).c_str() ), atof( params.at(8).c_str() ) );
        optimizer->SetWeight( atof( params.at(9).c_str() ) );
        optimizer->SetInertiaWeightType( atoi( params.at(10).c_str() ) );
        optimizer->SetRunNo( atoi( params.at(11).c_str() ) );
    }
    catch( ... )
    {
      cout<<"Za problem s kodom \""<<params.at(0)<<"\" parametri su pogrešno definirani."<<endl;
      continue;
    }
    
    
    // setting parameters/FunType and opening files for results
    if( !params.at(0).compare("SF6") )
    {
      optimizer->SetFunType( F6 );
      stringstream ss;
      ss<<++counterSF6;
      path=string( argv[3] ).append( "SF6_" ).append( ss.str() );
      fp=open_file( path, &mode );
    }
    else if( !params.at(0).compare("DJS") )
    {
      optimizer->SetFunType( SPHERE );
      stringstream ss;
      ss<<++counterDJS;
      path=string( argv[3] ).append( "DJS_" ).append( ss.str() );
      fp=open_file( path, &mode );
    }
    else if( !params.at(0).compare("RAS") )
    {
      optimizer->SetFunType( RASTRIGIN );
      stringstream ss;
      ss<<++counterRAS;
      path=string( argv[3] ).append( "RAS_" ).append( ss.str() );
      fp=open_file( path, &mode );
    }
    else if( !params.at(0).compare("GW") )
    {
      optimizer->SetFunType( GRIEWANGK );
      stringstream ss;
      ss<<++counterGW;
      path=string( argv[3] ).append( "GW_" ).append( ss.str() );
      fp=open_file( path, &mode );
    }
    else if( !params.at(0).compare("RB") )// RB
    {
      optimizer->SetFunType( ROSENBROCK );
      stringstream ss;
      ss<<++counterRB;
      path=string( argv[3] ).append( "RB_" ).append( ss.str() );
      fp=open_file( path, &mode );
    }
    
    if( !fp )
    {
      if( mode==YES || mode==YES_TO_ALL || mode==ASK_USER ) cout<<"Ne mogu napraviti datoteku \""<<path<<"\", preskačem"<<endl;
      else cout<<"Preskačem \""<<path<<"\""<<endl;
      continue;
    }
    
    
    optimizer->Optimize();
    
    optimizer->ExportToGnuPlot( fp );
    fclose(fp);
    cout<<"Upisao \""<<path<<"\"";
    optimizer->GenerateGnuFile( path );
    cout<<" i generirao \""<<path<<".gnu\""<<endl;
    delete optimizer;
  }
 
  // housework
  return 0;
}


vector<string> &Parse( const string &str, vector<string> &params )
{
  // housework
  params.clear();
  
  size_t first, second;
  bool finish=false;
  
  // first param - codename
  first=0;  
  second=str.find_first_of(" \t");
  
  
  do
  {
    if( second==string::npos )
    {
      second=str.length();
      finish=true;
    }
  
    params.push_back( str.substr( first, second - first ) );
    
    if( second==str.length() ) continue;
    
    first=second + 1;
    second=str.find_first_of( " \t", first );
  }
  while( !finish );
  
  return params;
}



FILE *open_file( string path, int *mode )
{
  FILE *fp=fopen( path.c_str(), "r" );
  if( !fp )
  {
    return fopen( path.c_str(), "w+" );
  }
  else
  {
    if( *mode!=YES_TO_ALL && *mode!=NO_TO_ALL )
    {
      for(;;)
      {
        cout<<"Datoteka \""<<path<<"\" postoji. Prebrisati? ((D)a za sve/(d)a/(n)e/(N)e za sve)";
        string a;
        cin>>a;
        if( !a.compare("d") || !a.compare("D") || !a.compare("y") || !a.compare("Y") )
        {
            if( !a.compare("D") || !a.compare("Y") ) *mode=YES_TO_ALL;
            else *mode=YES;
            return fopen( path.c_str(), "w+" );
        }
        else if( !a.compare("n") || !a.compare("N") )
        {
            if( !a.compare("N") ) *mode=NO_TO_ALL;
            else *mode=NO;
            return NULL;
        }
      }
    }
    else if( *mode==YES_TO_ALL ) return fopen( path.c_str(), "w+" );
    else if( *mode==NO_TO_ALL ) return NULL;
  }
}

