#include "particleswarmoptimizer.h"


// ParticleSwarmOptimizer::ParticleSwarmOptimizer():_swarm(NULL)
// {
//   SetDimension();
// }


int ParticleSwarmOptimizer::SetDimension( int d )
{
  _dimension=d;
  int sum=0;
  if( !_swarm ) return 0;   // swarm not initialized
  else                      // else set dimension for each particle
  {
    for( int i=0;i<_swarmSize;i++ )
    {
      sum+=_swarm[i].SetDimension( d );
    }
  }
  if( !sum ) return 0; // no memory allocation errors
  return -1;
}


int ParticleSwarmOptimizer::SetSwarmSize( int size )
{
  // housework
  if( _swarm )
  {
    delete[] _swarm;
    _swarm=NULL;
  }
  
  // allocate memory
  _swarm=new PParticle[size];
  if( !_swarm ) return -1;
  
  // set swarm size
  _swarmSize=size;
  
  // set dimension for each particle
  int sum=0;
  for( int i=0;i<size;i++ )
  {
    sum+=_swarm[i].SetDimension( _dimension );
  }
  if( !sum ) return 0; // no memory allocation errors
  return -2;
}


void ParticleSwarmOptimizer::SetRunNo( int runNo )
{
  if( runNo<1 ) _runNo=1;
  else _runNo=runNo;
}

void ParticleSwarmOptimizer::SetMaxX( float x )
{
  _maxX=x;
}

void ParticleSwarmOptimizer::SetMaxV( float v )
{
  _maxV=v;
}

int ParticleSwarmOptimizer::SetMaxIterations( int iter )
{
  _maxIter=iter;
  if( _results )
  {
    delete _results;
    _results=NULL;
  }
  _results=new float[iter];
  if( !_results ) return -1;
  if( _sums )
  {
    delete[] _sums;
    _sums=NULL;
  }
  _sums=new float[iter];
  if( !_sums )
  {
    delete[] _results;
    _results=NULL;
    return -1;
  }
  return 0;
}

void ParticleSwarmOptimizer::SetCutoff( float cutoff )
{
  _cutoff=cutoff;
}

void ParticleSwarmOptimizer::SetInitRang( float left, float right )
{
  _initRangLeft=left;
  _initRangRight=right;
}

void ParticleSwarmOptimizer::SetWeight( float weight )
{
  _weight=weight;
}

void ParticleSwarmOptimizer::SetFunType( int type )
{
  _funType=type;
}

void ParticleSwarmOptimizer::SetInertiaWeightType( int type )
{
  _weightType=type;
}


void ParticleSwarmOptimizer::RandomizeSwarm()
{
  for( int a=0;a<_swarmSize;a++ )
  {
    for( int b=0;b<_dimension;b++ )
    {
      _swarm[a].SetX( (float)rand()/(float)RAND_MAX * (_initRangRight-_initRangLeft) + _initRangLeft, b );
      _swarm[a].SetPbestx( _swarm[a].GetX( b ), b );
      _swarm[a].SetV( (float)rand()/(float)RAND_MAX*_maxV, b );
      
      // if ((rand()/32767.0) > 0.5) _swarm[a].SetV( -_swarm[a].GetV( b ), b );
    }
  }
}

void ParticleSwarmOptimizer::ExportToGnuPlot( FILE *fp )
{
    // write header
  fprintf( fp, "#\tX\tY\n" );
  
  // write data
  for( int i=0;i<_maxIter;i++ )
  {
    fprintf( fp, "\t%f\t%f\n", (float)i, _results[i] );
    if( _results[i]<=_cutoff ) break;
  }
}

int ParticleSwarmOptimizer::ExportToGnuPlot( string path )
{
  FILE *fp=fopen( path.c_str(), "w+" );
  if( !fp ) return -1;
  
  ExportToGnuPlot( fp );
  
  fclose(fp);
  
  return 0;
}

int ParticleSwarmOptimizer::GenerateGnuFile( string path )
{
  FILE *fp=fopen( string(path).append(".gnu").c_str(), "w+" );
  if( !fp ) return -1;
  
  float xpos=0.9;
  float ypos=0.85;
  float yspace=0.03;
  
  fprintf( fp, "reset\n" );
  fprintf( fp, "set xlabel \"Iteracije\"\n" );
  fprintf( fp, "set ylabel \"Najbolje od roja\"\n" );
  fprintf( fp, "set label 1 \"Funkcija (id): " );
  switch( _funType )
  {
    case F6:
      fprintf( fp, "Schaffer F6" );
      break;
      
    case SPHERE:
      fprintf( fp, "De Jongs' Sphere" );
      break;
      
    case ROSENBROCK:
      fprintf( fp, "Rosenbrock" );
      break;
      
    case RASTRIGIN:
      fprintf( fp, "Rastrigin" );
      break;
      
    case GRIEWANGK:
      fprintf( fp, "Griewangk" );
      break;
      
    case USER_DEFINED:
      fprintf( fp, "Korisnički definirana" );
    default:
      break;
  }
  fprintf( fp, " (%d)\" at screen %f, screen %f right\n", _funType, xpos, ypos ); ypos-=yspace;
  fprintf( fp, "set label 2 \"Broj dimenzija: %d\" at screen %f, screen %f right\n", _dimension, xpos, ypos ); ypos-=yspace;
  fprintf( fp, "set label 3 \"Broj cestica: %d\" at screen %f, screen %f right\n", _swarmSize, xpos, ypos ); ypos-=yspace;
  fprintf( fp, "set label 4 \"Max X: %f\"  at screen %f, screen %f right\n", _maxX, xpos, ypos ); ypos-=yspace;
  fprintf( fp, "set label 5 \"Max V: %f\"  at screen %f, screen %f right\n", _maxV, xpos, ypos ); ypos-=yspace;
  fprintf( fp, "set label 6 \"Rang lijevo: %f\" at screen %f, screen %f right\n", _initRangLeft, xpos, ypos ); ypos-=yspace;
  fprintf( fp, "set label 7 \"Rang desno: %f\" at screen %f, screen %f right\n", _initRangRight, xpos, ypos ); ypos-=yspace;
  fprintf( fp, "set label 8 \"Inercijska konstanta: %f\" at screen %f, screen %f right\n", _weight, xpos, ypos ); ypos-=yspace;
  fprintf( fp, "set label 9 \"Tip inercijske konstante: %d\" at screen %f, screen %f right\n", _weightType, xpos, ypos ); ypos-=yspace;
  fprintf( fp, "set label 10 \"Najveci broj iteracija: %d\" at screen %f, screen %f right\n", _maxIter, xpos, ypos ); ypos-=yspace;
  fprintf( fp, "set label 11 \"Cutoff: %f\" at screen %f, screen %f right\n", _cutoff, xpos, ypos ); ypos-=yspace;
  fprintf( fp, "plot \"%s\" with lines\n", path.c_str() );
  
  fclose(fp);
  return 0;
}


float ParticleSwarmOptimizer::f6( int particleIndex )
{
  if( _dimension<2 ) return 0.0f;
  
  double s, num, denom, denomt;
  float f;
  
  s=sin( sqrt( _swarm[particleIndex].GetX(0)*_swarm[particleIndex].GetX(0) + _swarm[particleIndex].GetX(1)*_swarm[particleIndex].GetX(1) ) );
  num=s*s-0.5;
  
  denomt=1.0 + 0.001*( _swarm[particleIndex].GetX(0)*_swarm[particleIndex].GetX(0) + _swarm[particleIndex].GetX(1)*_swarm[particleIndex].GetX(1) );
  denom=denomt*denomt;
  f=0.5+num/denom;
  
  return f;
}

float ParticleSwarmOptimizer::Sphere( int particleIndex )
{
  float result=0.0;
  for( int i=0;i<_dimension;i++ )
  {
    result+=_swarm[particleIndex].GetX(i)*_swarm[particleIndex].GetX(i);
  }
  return result;
}

float ParticleSwarmOptimizer::Rosenbrock( int particleIndex )
{
  float result=0.0;
  for( int i=1;i<_dimension;i++ )
  {
    result+=100.0*(_swarm[particleIndex].GetX(i) - _swarm[particleIndex].GetX(i-1)*_swarm[particleIndex].GetX(i-1) ) + (_swarm[particleIndex].GetX(i-1)-1)*(_swarm[particleIndex].GetX(i-1)-1);
  }
  
  return fabs( result );
}

float ParticleSwarmOptimizer::Rastrigin( int particleIndex )
{
  float result=0.0;
  for( int i=0;i<_dimension;i++ )
  {
    result+=_swarm[particleIndex].GetX(i)*_swarm[particleIndex].GetX(i) - 10.0*cos( 2.0*M_PI*_swarm[particleIndex].GetX(i) ) + 10.0;
  }
  return result;
}

float ParticleSwarmOptimizer::Griewangk( int particleIndex )
{
  float result_s=0.0;
  float result_p=1.0;
  
  for( int i=0;i<_dimension;i++ )
  {
    result_s+=_swarm[particleIndex].GetX(i)*_swarm[particleIndex].GetX(i);
    result_p*=cos( _swarm[particleIndex].GetX(i)/sqrt( i+1 ) );
  }
  
  return result_s/4000.0 + result_p - 1.0;
}

float ParticleSwarmOptimizer::UserFunction( int particleIndex )
{
  Sphere( particleIndex );
}
