#ifndef _PARTICLE_SWARM_OPTIMIZER_H_
#define _PARTICLE_SWARM_OPTIMIZER_H_

#include <string>
#include <cmath>
#include <cstdlib>
#include <cstdio>

#include "particle.h"

using namespace std;

enum
{
  F6,
  SPHERE,
  ROSENBROCK,
  RASTRIGIN,
  GRIEWANGK,
  USER_DEFINED
};

enum
{
  CONSTANT,
  TIME_VARIANT
};


class ParticleSwarmOptimizer 
{
  public:
    int SetDimension( int d=1 ); // default dimension=1, returns 0 on success
    int SetSwarmSize( int size ); // returns 0 on success, -1,-2 on error
    void SetRunNo( int runNo );
    void SetMaxX( float x );
    void SetMaxV( float v );
    int SetMaxIterations( int iter=10 );
    void SetCutoff( float cutoff );
    void SetInitRang( float left, float right );
    void SetWeight( float weight );
    void RandomizeSwarm();
    void SetFunType( int type );
    void SetInertiaWeightType( int type=TIME_VARIANT );
    void ExportToGnuPlot( FILE *fp );
    int ExportToGnuPlot( string path ); // returns 0 on success, -1 on error (cannot write file)
    
    int GenerateGnuFile( string path ); // returns 0 on success, -1 on error (cannot write file)
    
    virtual void Optimize() =0;
    
    float f6( int particleIndex );
    float Sphere( int particleIndex );
    float Rosenbrock( int particleIndex );
    float Rastrigin( int particleIndex );
    float Griewangk( int particleIndex );
    virtual float UserFunction( int particleIndex );
    
  protected:
    int _funType;
    float _cutoff;
    int _dimension;
    int _maxIter;
    float _maxV;
    float _maxX;
    int _runNo;
    PParticle *_swarm;
    int _swarmSize;
    float _initRangLeft;
    float _initRangRight;
    float _weight;
    int _weightType;
    float *_sums;
    float *_results;
};

#endif
