#include "rpsoptimizer.h"


RPSOptimizer::RPSOptimizer()
{
  _swarm=NULL;
  SetDimension();
  _results=NULL;
  _sums=NULL;
  SetMaxIterations();
  SetInertiaWeightType();
}

RPSOptimizer::~RPSOptimizer()
{
  delete[] _swarm;
  delete[] _results;
  delete[] _sums;
}


void RPSOptimizer::Optimize()
{
  RandomizeSwarm();
  
  int iter=0;
  int gbest=0; // initially assume the first particle as gbest
  
  float weight_up;
  float minival;
  
  bool finish=false;
  
  MMatrix tx( _dimension, _swarmSize );
  
  for( int i=0;i<_maxIter;i++ ) _sums[i]=0.0;
  
  for( int runs=0;runs<_runNo;runs++ )
  {
    do
    {
      if( iter >= 32760 ) iter=0; // so it doesn't crash the data type
        
        // update inertia weight
      switch( _weightType )
      {
        case TIME_VARIANT:
          weight_up=(_weight-0.4) * (_maxIter - iter) / _maxIter + 0.4; //time variant weight, linear from weight to 0.4
          break;
                
        case CONSTANT:
        default:
          weight_up=_weight; // constant inertia weight
          break;
      }
        
        
      for( int a=0;a<_swarmSize;a++ )
      {
        switch( _funType )
        {
          case F6:
            minival=f6( a );
            break;
                
          case SPHERE:
            minival=Sphere( a );
            break;
                
          case ROSENBROCK:
            minival=Rosenbrock( a );
            break;
                
          case RASTRIGIN:
            minival=Rastrigin( a );
            break;
                
          case GRIEWANGK:
            minival=Griewangk( a );
            break;
                
          case USER_DEFINED:
            minival=UserFunction( a );
            break;
                
          default:
            minival=UserFunction( a );
            break;
        }
            
        if( iter==0 ) _swarm[a].SetPbest( minival );
            
        if( minival < _swarm[a].GetPbest() )
        {
          _swarm[a].SetPbest( minival );
          for( int b=0;b<_dimension;b++ ) _swarm[a].SetPbestx( _swarm[a].GetX( b ), b );
                
          if( _swarm[a].GetPbest() < _swarm[gbest].GetPbest() ) gbest=a;
        }
            
        // asynchronous version
        for( int b=0;b<_dimension;b++ )
        {
          _swarm[a].SetV( weight_up*_swarm[a].GetV( b ) + 2*(rand()/(float)RAND_MAX)*(_swarm[a].GetPbestx( b )-_swarm[a].GetX( b )) +
              2*weight_up*(rand()/(float)RAND_MAX)*(_swarm[gbest].GetPbestx( b )-_swarm[a].GetX( b )) +
              2*weight_up*(rand()/(float)RAND_MAX), b ); // possibly set maxZ
                
          if( _swarm[a].GetV( b )>_maxV ) _swarm[a].SetV( _maxV, b );
          else if( _swarm[a].GetV( b )<-_maxV ) _swarm[a].SetV( -_maxV, b );
        }
            
            // tx allows simultaneous updates, cause positions are read by the functions we optimize so updating
            // positions here would affect the results and we would lose the ilusion of particles flying simultaneously
        for( int b=0;b<_dimension;b++ )
        {
          tx.Set( b, a, _swarm[a].GetX( b )+_swarm[a].GetV( b ) );
        }
      }
        
        // update positions
      for( int a=0;a<_swarmSize;a++ )
      {
        for( int b=0;b<_dimension;b++ )
        {
          _swarm[a].SetX( tx.Get( b, a ), b );
        }
      }
        
      _sums[iter]+=_swarm[gbest].GetPbest();
        
      if( _swarm[gbest].GetPbest()<=_cutoff || iter>=_maxIter ) finish=true;
      iter++;
    }
    while( !finish );
  }
  
  for( int i=0;i<iter && i<_maxIter;i++ )
  {
    _results[i]=_sums[i]/_runNo;
  }
}
